extern crate hercules_rt_proc;

pub(crate) mod elf;
pub(crate) use crate::elf::*;
pub use hercules_rt_proc::use_hbin;

#[derive(Debug)]
pub struct Module {
    elf: Elf,
}

impl Module {
    /*
     * Parse ELF object from ELF bytes buffer.
     */
    pub fn new(buffer: &[u8]) -> Self {
        let elf = unsafe { parse_elf(buffer) };
        Module { elf }
    }

    /*
     * Get the function pointer corresponding to a function name. Panic if not
     * found.
     */
    pub unsafe fn get_function_ptr(&self, name: &str) -> *mut u8 {
        self.elf.program_section.offset(
            self.elf.function_pointers[self
                .elf
                .function_names
                .iter()
                .position(|s| s == name)
                .unwrap()],
        )
    }
}
